package com.hello2morrow.sonargraph.languageprovider.java.command.system;

import static org.junit.Assert.assertEquals
import static org.junit.Assert.assertFalse
import static org.junit.Assert.assertNotNull
import static org.junit.Assert.assertTrue

import org.junit.Rule
import org.junit.Test
import org.junit.rules.TemporaryFolder

import com.hello2morrow.foundation.utilities.OperationResult
import com.hello2morrow.foundation.utilities.OperationResultWithOutcome
import com.hello2morrow.sonargraph.core.command.system.ExportMetaDataCommand
import com.hello2morrow.sonargraph.core.command.system.ExportMetaDataCommand.ExportData
import com.hello2morrow.sonargraph.core.model.analysis.IMetricId
import com.hello2morrow.sonargraph.core.model.common.Severity
import com.hello2morrow.sonargraph.core.model.metrics.CoreMetricId
import com.hello2morrow.sonargraph.core.model.metrics.MetricCategory
import com.hello2morrow.sonargraph.core.model.script.ScriptMetricId
import com.hello2morrow.sonargraph.core.model.system.SoftwareSystemState
import com.hello2morrow.sonargraph.languageprovider.java.foundation.common.JavaTestFixture
import com.hello2morrow.sonargraph.languageprovider.java.model.metrics.JavaMetricId

import de.schlichtherle.truezip.file.TFile
import de.schlichtherle.truezip.file.TFileInputStream

public class ExportMetricMetaDataCommandTest extends AbstractJavaCommandTest {
    private static final String PROJECT_PATH = JavaTestFixture.ALARMCLOCK_WITH_ARCHITECTURE;
    private static final String EXPORT_FILE_NAME = "MetricData.xml";
    private static final double DELTA = 0.0001;

    private OperationResult m_result;

    @Rule
    public TemporaryFolder m_outputFolder = new TemporaryFolder();

    public ExportMetricMetaDataCommandTest() {
        super(PROJECT_PATH);
    }

    @Test
    public void testExport() {
        final TFile targetFile = new TFile(m_outputFolder.getRoot(), EXPORT_FILE_NAME);
        assertFalse("output file must not exist", targetFile.exists());

        openAndClearAndRefreshSystem(new TFile(PROJECT_PATH));
        assertTrue("No software system available", getController().hasSoftwareSystem());
        assertEquals("Wrong system state", SoftwareSystemState.MODEL_LOADED, getController().getSoftwareSystem().getState());

        final List<IMetricId> metricIdsToExport = new ArrayList<>();
        metricIdsToExport.add(CoreMetricId.CORE_ACD);
        metricIdsToExport.add(CoreMetricId.CORE_VIOLATIONS_PARSER_DEPENDENCIES);
        metricIdsToExport.add(JavaMetricId.JAVA_PACKAGES);
        metricIdsToExport.add(new ScriptMetricId("./Core/BadSmells/UnusedTypes.scr", "Unused Types", "Number of Unused Types",
                "Number of types with no incoming references", false, 0, 0, Severity.WARNING, MetricCategory.values()));

        final ExportMetaDataCommand export = new ExportMetaDataCommand(getController(),
                new ExportMetaDataCommand.IExportInteraction() {
                    @Override
                    public boolean collect(final ExportData data) {
                        data.setTargetFile(targetFile);
                        data.setMetricIds(metricIdsToExport);
                        return true;
                    }

                    @Override
                    public void processResult(final OperationResultWithOutcome<TFile> result) {
                        m_result = result;
                    }
                });
        export.runSynchronously();
        assertNotNull("Result must not be null!", m_result);
        assertTrue(m_result.toString(), m_result.isSuccess());

        assertTrue("Output file does not exist", targetFile.exists());

        def metaDataXml = new XmlSlurper().parse(new TFileInputStream(targetFile));
        assertEquals("wrong root element", 'exportMetaData', metaDataXml.name());
        assertEquals("wrong version", getController().getInstallation().getVersion().toString(), metaDataXml.@version.toString());
        assertEquals("Unexpected number of elements", 7, metaDataXml.children().size());

        assertEquals("wrong issueprovider name", "./Core/BadSmells/UnusedTypes.scr", metaDataXml.issueProviders.issueProvider[0].@name.toString());
        assertEquals("wrong issueprovider name", "JavaLanguageProvider", metaDataXml.issueProviders.issueProvider[5].@name.toString());

        assertEquals("wrong issuecategory name", "ArchitectureConsistency", metaDataXml.issueCategories.category[0].@name.toString());
        assertEquals("wrong issuecategory name", "ArchitectureViolation", metaDataXml.issueCategories.category[2].@name.toString());
        assertEquals("wrong issuecategory name", "Workspace", metaDataXml.issueCategories.category[14].@name.toString());

        assertEquals("Unexpected number of issue types", 68, metaDataXml.issueTypes.children().size());
        assertEquals("wrong issueType name", "UnresolvedRequiredArtifact", metaDataXml.issueTypes.issueType[0].@name.toString());

        assertEquals("wrong provider name", "Core", metaDataXml.metricProviders.provider[0].@name.toString());
        assertEquals("wrong provider name", "JavaLanguageProvider", metaDataXml.metricProviders.provider[1].@name.toString());
        assertEquals("wrong provider name", "./Core/BadSmells/UnusedTypes.scr", metaDataXml.metricProviders.provider[2].@name.toString());//

        // TODO: AvZ: Always fails when new categories are added, test must be improved

        //        assertEquals("wrong category name", "Architecture", metaDataXml.metricCategories.category[0].@name.toString());
        //        assertEquals("wrong category name", "Size", metaDataXml.metricCategories.category[1].@name.toString());
        //        assertEquals("wrong category name", "JohnLakos", metaDataXml.metricCategories.category[2].@name.toString());
        //        assertEquals("wrong category name", "Script", metaDataXml.metricCategories.category[3].@name.toString());

        assertEquals("wrong metricLevel name", "System", metaDataXml.metricLevels.level[0].@name.toString());
        assertEquals("wrong order number", 0, metaDataXml.metricLevels.level[0].@orderNumber.toInteger());
        assertEquals("wrong metricLevel name", "Module", metaDataXml.metricLevels.level[1].@name.toString());
        assertEquals("wrong order number", 10, metaDataXml.metricLevels.level[1].@orderNumber.toInteger());

        assertEquals("wrong metricId name", "CoreAcd", metaDataXml.metricIds.metricId[0].@name.toString());
        assertEquals("wrong bestValue", 0.0, new Double(metaDataXml.metricIds.metricId[0].@bestValue.toString()), DELTA);
        //String comparison needed here
        assertEquals("wrong worstValue", "INF", metaDataXml.metricIds.metricId[0].@worstValue.toString());

        assertEquals("wrong metricId name", "CoreViolationsParserDependencies", metaDataXml.metricIds.metricId[1].@name.toString());

        assertEquals("wrong metricId name", "JavaPackages", metaDataXml.metricIds.metricId[2].@name.toString());
        //String comparisons needed here
        assertEquals("wrong bestValue", "", metaDataXml.metricIds.metricId[2].@bestValue.toString());
        assertEquals("wrong worstValue", "", metaDataXml.metricIds.metricId[2].@bestValue.toString());

        assertEquals("wrong metricId name", "Unused Types", metaDataXml.metricIds.metricId[3].@name.toString());
    }
}